function [MaxOptid,PrODmodel,xThresh] =   PrOD(OutOptid,dMax,wvl,dCal,pixCal)
% Fit logistic regression model for identifying optically deep water and apply
%
%% PrOD.m:
%   Function for fitting a logistic regression model to identify optically deep
%   water.  Takes as input the results from Optimal Band Ratio Analysis (OBRA)
%   of Progressively Truncated Input Depths (OPTID), including the maximum
%   detectable depth inferred by the user.  Uses this information to calculate
%   values of the image-derived quantity X for the calibration pixels provided
%   as input.  The corresponding depths are then used to partitition the
%   calibration data set into two subsets, one with depths greater than dMax and
%   the other with depths less than dMax.  The logistic regression model
%   distinguishes between these two classes by calculating the probability of
%   observing optically deep water, Pr(OD), based on the calibration pixel
%   values and depths.  This function returns the Pr(OD) model and the threshold
%   value of X for a default probability cutoff of 0.5.  Figures illustrating
%   the distribution of the shallow and deep classes and the logistic regression
%   model are produced as well.
%
%% SYNTAX:
%   [MaxOptid,PrODmodel,xThresh] =   PrOD(OutOptid,dMax,wvl,dCal,pixCal);   
%
%% INPUTS:
%   OutOptid:   Output structure array with OPTID results, produced using the
%               function GenOptidLin. The key fields called in this funtion are
%               the list of cutoff depths in dCutList and the numerator and
%               denominator wavelengths of an exponential X vs. d relation in
%               expNumWvl and expDenWvl
%   dMax:       Maximum detectable depth (m) inferred by the user based on the
%               OPTID plot of OBRA R-squared vs. cutoff depth
%   wvl:        List of wavelengths (nm) in the image
%   dCal:       Depths measured at the locations of the calibration pixels (m)
%   pixCal:     Spectra from calibration pixels in an nSpectra X nWvl array
%   
%% OUTPUTS:
%   MaxOptid:   OPTID output for the depth cutoff inferred to represent dMax
%   PrODmodel:  Logistic regression model for estimating the probability of
%               optically deep water, Pr(OD). This is a GeneralizedLinearModel
%               object, so see the MATLAB help for more details on properties
%               and methods associated with this type of object
%   xThresh:    Threshold value of the image-derived quantity X for
%               distinguishing shallow vs. optically deep water, based on an 
%               assumed Pr(OD) cutoff of 0.5.
%   Figure showing histograms of X for the relatively shallow vs. optically deep
%   water clases for the calibration data set
%   Figure showing the fitted logistic regression model, input data, confidence
%   intervals, and threshold value of X for a cutoff of Pr(OD) = 0.5
%   
%% NOTES:
%   > This function calls the function fitglm.m from the MATLAB Statistics and
%   Machine Learning Toolbox, which thus is required to run this function. See
%   the help on fitglm.m for more details.
%   > See also GenOptidLin.m and GenObraLin.m for the OPTID analysis from which
%   the maximum detectable depth is inferred
%   > A model slice plot is produced but this function extracts the data from
%   this plot and reformats in a separate figure, so you can ignore the model
%   slice plot.
%
%% FUNCTION SUMMARY:
%   [MaxOptid,PrODmodel,xThresh] =   PrOD(OutOptid,dMax,wvl,dCal,pixCal);

%% CREDITS:
% Carl J. Legleiter - cjl@usgs.gov
% 12/10/2018
% C:\Carl\CJLmfiles\RS\PrOD.m

%% Get OBRA wavelengths for dMax identified via OPTID
% Get list of cutoff depths
dCutList    =   [OutOptid.dCut]';
% Get the index that corresponds to the inferred dMax
iDmax       =   dCutList == dMax;
% Get the OPTID output for this cutoff depth
MaxOptid    =   OutOptid(iDmax);
% Numerator and denominator bands
numWvl      =   MaxOptid.expNumWvl;
denWvl      =   MaxOptid.expDenWvl;
% % Coefficients of exponential X vs. d relation
% constCoeff  =   MaxOptid.ExpObra.constCoeff;
% expCoeff    =   MaxOptid.ExpObra.expCoeff;


%% Examine distribution of X for pixels above and below Dmax
% Define X using the wavelengths identified by OBRA for data up to dMax
Xcal        =   log(pixCal(:,wvl==numWvl)./pixCal(:,wvl==denWvl));
% Create a plot showing histograms of X values for points shallower than dMax
% and points deeper than dMax
figure
histogram(Xcal(dCal<=dMax),'Normalization','probability','FaceColor',[1 0 0],...
          'BinWidth',range(Xcal)/20);
hold on
histogram(Xcal(dCal>dMax),'Normalization','probability','FaceColor',[0 0 1],...
          'BinWidth',range(Xcal)/20);
legend('d < d_{max}','d > d_{max}')
xlabel(['X = ln[R(' num2str(numWvl,3) ')/R(' num2str(denWvl,3) ')]'])
ylabel('Relative probability')
title('Distributions of X for depths above or below OPTID-based d_{max}')
   
   
%% Fit logistic regression model to distinguish areas above and below dMax
% Create a binary response variable with 1's for pixels with depths greater than
% the dMax inferred via OPTID
isDeepCal   =   dCal>=dMax;
% Our only predictor variable, which is continuous, is X
% See the MATLAB help for fitglm. Based on the example, to fit a logistic
% regression model we just have to specify the distribution as binomial
PrODmodel   =   fitglm(Xcal,isDeepCal,'Distribution','binomial');
    

%% Calculate threshold value of X for distinguishing optically deep water
%   x = -1/B1{ln[1/p(x) - 1] + B0}
% Assume a probability cutoff of 0.5
pCut        =   0.5;
xThresh     =   -1/PrODmodel.Coefficients.Estimate(2)*(log(1/pCut - 1) + ...
                                                 PrODmodel.Coefficients.Estimate(1));

                                             
%% Plot logistic regression model
h   =   plotSlice(PrODmodel);
% Dig through the property hierarchy to identify which handles correspond to the
% lines on the modelSlice plot so we can create a new figure with thse data
h.Children(end).Children(1);
    % This is the horizontal dotted line for the example probability, so we'll
    % want to just use a fixed value of 0.5 with the hline function
h.Children(end).Children(2);
    % This is the vertical dashed line for the example X value for the
    % probability above, so we'll want to just use a fixed value of xThresh
    % (calculated previously) with the vline function
h.Children(end).Children(3);
    % This is one of the probability curve confidence limits, so we can extract
    % the x and y data to plot this confidence interval
xLoBound    =   h.Children(end).Children(3).XData;
yLoBound    =   h.Children(end).Children(3).YData;
h.Children(end).Children(4);
    % This is one of the probability curve confidence limits, so we can extract
    % the x and y data to plot this confidence interval
xHiBound    =   h.Children(end).Children(4).XData;
yHiBound    =   h.Children(end).Children(4).YData;
h.Children(end).Children(5);
    % This is the probability curve itself, so we can extract the x and y data
    % to plot this line
xCurve      =   h.Children(end).Children(5).XData;
yCurve      =   h.Children(end).Children(5).YData;
h.Children(end).Children(6);
    % This is the actual data used to produce the model, so we can extract the x
    % and y data to plot the data as points
xData       =   h.Children(end).Children(6).XData;
yData       =   h.Children(end).Children(6).YData;
xData0      =   xData(yData == 0);
yData0      =   yData(yData == 0);
xData1      =   xData(yData == 1);
yData1      =   yData(yData == 1);

% Now make a new figure with a cleaned up version of this plot
figure
plot(xData0,yData0,'rs');
hold on
plot(xData1,yData1,'bo');
plot(xCurve,yCurve,'b-');
plot(xLoBound,yLoBound,'b:',xHiBound,yHiBound,'b:');
hline(0.5,'k:');
vline(xThresh,'k:')
xlabel('X = ln[R(487)/R(587)]')
ylabel('Pr(OD)')
title('Logistic regression model for estimating probability of optically deep water')
legend('Shallow data','Optically deep data','Logistic regression model',...
       'Confidence interval','Confidence interval','location','northwest')
   
   
%% EXAMPLE CODE FOR VALIDATION, MASKING, AND PLOTTING BELOW ...


% %% Apply logistic regression model to validation data and map Pr(d >= dMax)
% % The model object has a predict method we can use to obtain probabilities for
% % new data, our validation points in this case
% Xval        =   log(pixValSub(:,wvl==numWvl)./pixValSub(:,wvl==denWvl));
% isDeepVal   =   predict(PrODmodel,Xval);
% figure
% ax1         =    subplot(1,2,1);
% scatter(xValSub,yValSub,10,dValSub); axis equal; resetMapTicks(100); box on
% colormap(ax1,flipud(jet)); colorbar; set(ax1,'clim',[0 max(dCal)])
% xlabel('Easting (m)');   ylabel('Northing (m)');
% title('Field-based validation depths (m)')
% ax2         =    subplot(1,2,2);
% scatter(xValSub,yValSub,10,isDeepVal); axis equal; resetMapTicks(100); box on
% colormap(ax2,gray); colorbar; set(ax2,'clim',[0 1])
% xlabel('Easting (m)');   ylabel('Northing (m)');
% title(['Pr(OD) for d_{max} inferred via OPTID: ' num2str(dMax) ' m'])
% 
% 
% %% Use validation subset to quantify classification accuracy for a given probability cutoff
% % We can use the MATLAB function confusionmat to obtain a confusion matrix and
% % then use that result to compute the total percent correctly classified.  Note
% % that the function confusionchart can be used to produce graphical output and
% % further results for the confusion matrix.
% isDeepCalPred   =   predict(PrODmodel,Xcal);
% isDeepValPred   =   predict(PrODmodel,Xval);
% pCut            =   0:0.01:1;
% pctCorrectCal   =   zeros(size(pCut));
% pctCorrectVal   =   zeros(size(pCut));
% trueDeepCal     =   dCal > dMax;
% trueDeepVal     =   dValSub > dMax;
% for i = 1:length(pCut)
%     predictedDeepCal    =   isDeepCalPred > pCut(i);
%     C                   =   confusionmat(trueDeepCal,predictedDeepCal);
%     pctCorrectCal(i)    =   sum(diag(C))/sum(sum(C));
%     predictedDeepVal    =   isDeepValPred > pCut(i);
%     C                   =   confusionmat(trueDeepVal,predictedDeepVal);
%     pctCorrectVal(i)    =   sum(diag(C))/sum(sum(C));
% end
% figure
% plot(pCut,pctCorrectCal,'r-',pCut,pctCorrectVal,'b-')
% xlabel('Probability cutoff')
% ylabel('Percent correctly classified as deep')
% legend('Calibration data','Validation data')
% title('Deep water classification accuracy: Kootenai River MBES site #5')
% 
% 
% %% Apply depth retrieval and probability of optically deep results to image
% % First calculate X values for each pixel in the image, using the numerator
% % and denominator wavelengths identified as optimal in the OBRA for dMax
% % inferred by OPTID
% Ximg    =   log(imgWaterFilt(:,:,wvl==numWvl)./imgWaterFilt(:,:,wvl==denWvl));
% % Now apply the X vs. d calibration relation to the image of X values
% dMap    =   constCoeff.*exp(expCoeff.*Ximg);
% imshowWater(dMap,waterMask,1,100,imXdata,imYdata)
% colormap(flipud(jet));
% set(gca,'clim',[0 max(dEchocal)])
% title('Kootenai River CASI image: Exponential OBRA depth retrieval (m)','fontsize',14)
% resetMapTicks(100)
% % Note that you can use impixelinfo to inspect the actual values in this map
% % impixelinfo
% 
% % Also predict the probabilty of optically deep water, which we'll call PrOD,
% % using our logistic regression model
% prodMap =   predict(PrODmodel,Ximg(:));
% prodMap =   reshape(prodMap,size(Ximg));
% % Display the resulting map
% imshowWater(prodMap,waterMask,1,100,imXdata,imYdata)
% colormap(gray);
% set(gca,'clim',[0 1])
% title('Kootenai River CASI image: Probability of optically deep water','fontsize',14)
% resetMapTicks(100)
% 
% 
% %% Produce a mask from the PrOD map for the specified probability cutoff: 0.5
% prodMask    =   buildWaterMask1band(prodMap,imXdata,imYdata,1);
% MaskPoly    =   mask2vector(prodMask,imXdata,imYdata);
% 
% 
% %% Calculate proportion of image with PrOD > 0.5
% % We just need the number of 1's in the mask for PrOD > 0.5 and the number of
% % 1's in the original water mask and we can obtain these counts by summing
% propDeep    =   sum(prodMask(:))/sum(waterMask(:));
% % 0.6421 for this reach
% 
% 
% %% Fuse the depth map and PrOD image in a single display, using PrOD to set transparency
% % We want the shallow areas in which we are more confident to appear more
% % opaque, so what we really need is 1 - PrOD as the alphadata
% % imshowWater(dMap,waterMask.*(1-prodMap),1,100,imXdata,imYdata)
% close all
% imshowWater(dMap,waterMask,1,100,imXdata,imYdata);
% colormap(flipud(jet));
% set(gca,'clim',[0 dMax])
% ax1     =   gca;
% % title('Kootenai River CASI image: Exponential OBRA depth retrieval (m)','fontsize',14)
% title('Kootenai River site #5: OPTID bathymetry and probability of optically deep water ','fontsize',14)
% hCbar1 =    findobj('type','colorbar');
% set(get(hCbar1,'Label'),'String','Depth (m)');
% % % Overlay the vector mask derived from the PrOD map
% % hold on
% % plot([MaskPoly.X],[MaskPoly.Y],'b-','linewidth',4)
% resetMapTicks(100)
% 
% 
% %% That looks good, so how about overlaying the PrOD map in grayscale
% ax2     =   axes('Position',get(ax1,'Position'));
% imagesc(imXdata,imYdata,prodMap,'alphadata',prodMask); axis equal; axis xy
% colormap(ax2,flipud(gray)); set(gca,'clim',[0.5 1]); 
% hCbar2  =   colorbar('EastOutside');
% set(get(hCbar2,'Label'),'String','Pr(d \geq d_{max})');
% set(hCbar2,'position',get(hCbar1,'position')+[0.05 0 0 0])
% title([])
% axis off
% axes(ax1)
% axes(ax2)