function [LinObra,QuadObra,ExpObra,PowerObra] = genObraLin(spectra,wvl,d,fitFlags,showPlot)
% Generalized Optimal Band Ratio Analysis (OBRA) for spectrally based depth retrieval
%
%% genObraLin.m:
%   Performs a generalized version of Optimal Band Ratio Analysis (OBRA) for
%   spectrally based depth retrieval from passive optical remotely sensed data.
%   For each possible combination of bands, the log-transformed band ratio is
%   related to the measured depths corresponding to the spectral data using
%   linear, quadratic (second-order polynomial), exponential, and power law
%   functions. For each type of model, the optimal band ratio is that
%   combination of bands that yields the highest coefficient of determination
%   R-squared.  The equations are fit using ordinary least squares linear
%   regression, with the exponential and power law models transformed to
%   linearity prior to regression.  Results are reported in the command window
%   and stored in a structure array for each model type.  Figures with
%   calibration scatter plots and OBRA matrices are produced for each model type
%   as well.  Performance is summarized in terms of R-squared and root mean
%   squared error (RMSE) values.
%
%% SYNTAX:
%   [LinObra,QuadObra,ExpObra,PowerObra] = genObraLin(spectra,wvl,d,fitFlags,showPlot);
%
%% INPUTS:
%   spectra =   nSpectra X nWvl matrix of spectral data (i.e., field
%               reflectance spectra or image data)
%   wvl     =   nWvl vector of wavelengths for the spectra
%   d       =   nSpectra vector of corresponding depth measurements in meters
%   fitFlags=   4 X 1 logical vector of flags for fitting the various types of
%               model; for each model type, use a 1 to fit or a 0 to not fit;
%               the order of the models is: [linear quadratic exponential power]
%               for example, to fit linear, quadratic, and power models, but not
%               exponential, use:   fitFlags    =   [1 1 0 1];
%   showPlot=   logical flag to produce OBRA matrix and X vs. d calibration
%               plots: 1 to plot, 0 to not
%
%% OUTPUTS:
%   LinObra =   Structure array containing the linear OBRA results in fields:
%       numWvl:     numerator wavelength of the optimal band ratio
%       denWvl:     denominator wavelength of the optimal band ratio
%       intercept:  constant coefficient for linear fit
%       slope:      slope coefficient for linear fit
%       R2:         R-squared value for the optimal X vs. d regression
%       rmse:       root mean square error for the optimal X vs. d regression
%   Figures displaying a calibration scatter plot and a matrix of R-squared
%   values for all possible wavelength combinations
%   QuadObra=   Structure array containing the quadratic OBRA results in fields:
%       numWvl:     numerator wavelength of the optimal band ratio
%       denWvl:     denominator wavelength of the optimal band ratio
%       intercept:  constant coefficient for quadratic fit
%       linTerm:    coefficient for linear term in quadratic fit
%       quadTerm:   coefficient for X^2 term in quadratic fit
%       R2:         R-squared value for the optimal X vs. d regression
%       rmse:       root mean square error for the optimal X vs. d regression
%   Figures displaying a calibration scatter plot and a matrix of R-squared
%   values for all possible wavelength combinations
%   ExpObra =   Structure array containing the exponential OBRA results in fields:
%       numWvl:     numerator wavelength of the optimal band ratio
%       denWvl:     denominator wavelength of the optimal band ratio
%       constCoeff: constant coefficient for exponential fit
%       expCoeff:   coefficient in exponential term in exponential fit
%       R2:         R-squared value for the optimal X vs. d regression
%       rmse:       root mean square error for the optimal X vs. d regression
%   Figures displaying a calibration scatter plot and a matrix of R-squared
%   values for all possible wavelength combinations
%   PowerObra =   Structure array containing the power law OBRA results in fields:
%       numWvl:     numerator wavelength of the optimal band ratio
%       denWvl:     denominator wavelength of the optimal band ratio
%       constCoeff: constant coefficient for power law fit
%       powerCoeff: coefficient with power X is raised to in power law fit
%       R2:         R-squared value for the optimal X vs. d regression
%       rmse:       root mean square error for the optimal X vs. d regression
%   Figures displaying a calibration scatter plot and a matrix of R-squared
%   values for all possible wavelength combinations
%
%% NOTES:
% * This function does not perform any spectral subsetting or interpolation, so
%   these operations will need to be done before using this code.  
% * Some example code for implementing this approach with the non-linear
%   regression functions from MATLAB's Statistics Toolbox is in a comment
% * Based on the original code in obra.m
% 
%% FUNCTION SUMMARY:
%   [LinObra,QuadObra,ExpObra,PowerObra] = genObraLin(spectra,wvl,d,fitFlags,showPlot);

%% CREDITS:
% Dr. Carl J. Legleiter, cjl@usgs.gov            
% Geomorphology and Sediment Transport Laboratory
% United States Geological Survey                
% 03/26/2018
% C:\Carl\CJLmfiles\RS\genObraLin.m


%% Use the nchoosek function to get all possible pairs of band indices
bandPairs   =   nchoosek((1:length(wvl))',2);
numWvlVec   =   wvl(bandPairs(:,1));
denWvlVec   =   wvl(bandPairs(:,2));


%% Pre-allocate some square matrices to hold the results for visualization
numWvl      =   zeros(length(wvl));
denWvl      =   zeros(length(wvl));
linR2       =   zeros(length(wvl));
quadR2      =   zeros(length(wvl));
expR2       =   zeros(length(wvl));
powerR2     =   zeros(length(wvl));


%% Loop over wavelength pairs and use regress to fit linear, quadratic, exponential, and power models
for iPair = 1:length(bandPairs)
    iNum    =   find(wvl==numWvlVec(iPair),1,'first');
    iDen    =   find(wvl==denWvlVec(iPair),1,'first');
    ratio   =   log(spectra(:,iNum)./spectra(:,iDen));
    % Use these results to populate the symmetric, square output matrices
    numWvl(bandPairs(iPair,1),bandPairs(iPair,2))   =   numWvlVec(iPair);
    denWvl(bandPairs(iPair,1),bandPairs(iPair,2))   =   denWvlVec(iPair);
    if fitFlags(1)
        [~,~,~,~,stats]     =   regress(d,[ones(size(ratio)) ratio]);
        linR2(bandPairs(iPair,1),bandPairs(iPair,2))    =   stats(1);
    end
    if fitFlags(2)
        [~,~,~,~,stats]     =   regress(d,[ones(size(ratio)) ratio ratio.^2]);
        quadR2(bandPairs(iPair,1),bandPairs(iPair,2))   =   stats(1);
    end
    if fitFlags(3) % Transform exponential model to linearity
        logD                =   log(d);
        [~,~,~,~,stats]     =   regress(logD,[ones(size(ratio)) ratio]);
        expR2(bandPairs(iPair,1),bandPairs(iPair,2))    =   stats(1);
        % Confirm with a plot
        % figure
        % plot(ratio,logD,'.')
        % hold on
        % plot(linspace(0,0.35,1000),exp(b(1))*exp(b(2)*linspace(0,0.35,1000)),'g-')
        % Replace any R2 values outside the range of [0 1] with NaN's
        expR2(expR2 >= 1)                               =   NaN;
    end
    if fitFlags(4)
        logD                =   log(d);
        logRatio            =   log(ratio);
        [~,~,~,~,stats]     =   regress(logD,[ones(size(logRatio)) logRatio]);
        powerR2(bandPairs(iPair,1),bandPairs(iPair,2))  =   stats(1);        
%         % Confirm with a plot
%         figure
%         plot(ratio,d,'.')
%         hold on
%         plot(linspace(0,0.35,1000),exp(b(1))*linspace(0,0.35,1000).^b(2),'g-')
        % Replace any R2 values outside the range of [0 1] with NaN's
        powerR2(powerR2 >= 1)                               =   NaN;
    end
end


%% NOTE: Example syntax for using fitnlm from the MATLAB statistics toolbox
% This would work, too, but ties us to a proprietary toolbox, so let's use the
% open-source ezfit instead
% fun =   @(b,x)b(1)*exp(b(2)*x);
% b0  =   [1 1];
% mdl =   fitnlm(x,y,fun,b0);
% mdl.Rsquared.ordinary
% mdl.Rsquared.adjusted
% mdl.Coefficients.Estimate

% Within the loop
%     ExpFit2 =   fitnlm(ratio,d,fun,b0);
%     expR22(bandPairs(iPair,1),bandPairs(iPair,2))   =   ExpFit2.Rsquared.ordinary;


%% Cycle through fit types, produce plots, and compile results (R2, coefficients)


%%  1) Linear
if fitFlags(1)
    [r2max,iMax]=   max(linR2(:));
    % This will give you the corresponding matrix indices
    [i,j]       =   ind2sub(size(linR2),iMax);
    numWvlMax   =   numWvl(i,j);
    denWvlMax   =   denWvl(i,j);
    % Repeat the fit rather than storing all the coefficients for band ratios we'll
    % never use
    ratio       =   log(spectra(:,wvl==numWvlMax)./spectra(:,wvl==denWvlMax));
    b           =   regress(d,[ones(size(ratio)) ratio]);
    disp('  ')
    disp('LINEAR OBRA')
    disp(['     Maximum R-squared: ' num2str(r2max)])
    disp(['     Optimal band ratio: log(R' num2str(numWvlMax) '/R' num2str(denWvlMax) ')'])
    disp(['     X vs. d relation: d = ' num2str(b(1),4) ' + ' ...
                num2str(b(2),4) 'X'])

    % Calculate predicted depths and then RMSE
    dPred       =   b(1)+b(2)*ratio;
    rmse        =   sqrt(mean((d-dPred).^2));

    if showPlot
        % Calibration plot
        figure
        plot(ratio,d,'b.');
        axLim       =   axis;
        hold on
        plot(axLim(1:2),b(1)+b(2)*axLim(1:2),'k-','linewidth',2);
        axis(axLim);
        text(0.1,0.9,char(['d = ' num2str(b(1),4) ' + ' num2str(b(2),4) 'X '],...
                          ['R^2 = ' num2str(r2max,3) '; RMSE = ' num2str(rmse,4) ' m']),...
                    'units','normalized','fontname','arial','fontsize',12)
        xlabel(['X = log(R' num2str(numWvlMax,3) '/R' num2str(denWvlMax,3) ')'])
        ylabel('Depth (m)')
        title('Linear OBRA calibration')

        % Plot the OBRA matrix
        xText       =   450;
        yText       =   650;
        figure
        mask                =   linR2;
        mask(isnan(linR2))  =   0;
        mask                =   logical(mask);
        imagesc(wvl,wvl,linR2,'alphadata',mask); axis image; colormap(jet);
        set(gca,'ydir','rev','clim',[0 1])
        box on; axis square
        set(gca,'fontname','arial','fontsize',12)
        xlabel('\lambda_2 (nm)')
        ylabel('\lambda_1 (nm)')
        title('OBRA: linear X vs. d relation','fontsize',13)
        hCbar       =   colorbar('location','EastOutside');
        set(get(hCbar,'title'),'string',{'OBRA','R^2'})
        str =   char(['\lambda_1 = ' num2str(numWvlMax,3) ' nm'],...
                    ['\lambda_2 = ' num2str(denWvlMax,3) ' nm'],...
                    ['d = ' num2str(b(1),4) ' + ' num2str(b(2),4) 'X'],...
                    ['R^2 = ' num2str(r2max,3) '; RMSE = ' num2str(rmse,3) ' m']);
        text(xText,yText,str,'fontsize',12,'fontname','arial','horizontalalignment','left',...
                'verticalalignment','top');
    end
        
    % Put these results in an output structure array
    LinObra =   struct( 'numWvl',numWvlMax,'denWvl',denWvlMax,...
                        'intercept',b(1),'slope',b(2),...
                        'R2',r2max,'rmse',rmse);
else
    LinObra =   [];
end


%%  2) Quadratic
if fitFlags(2)
    [r2max,iMax]=   max(quadR2(:));
    % This will give you the corresponding matrix indices
    [i,j]       =   ind2sub(size(quadR2),iMax);
    numWvlMax   =   numWvl(i,j);
    denWvlMax   =   denWvl(i,j);
    
    % Repeat the fit rather than storing all the coefficients for band ratios we'll
    % never use
    ratio       =   log(spectra(:,wvl==numWvlMax)./spectra(:,wvl==denWvlMax));
    b           =   regress(d,[ones(size(ratio)) ratio ratio.^2]);
    disp('  ')
    disp('QUADRATIC OBRA')
    disp(['     Maximum R-squared: ' num2str(r2max)])
    disp(['     Optimal band ratio: log(R' num2str(numWvlMax) '/R' num2str(denWvlMax) ')'])
    disp(['     X vs. d relation: d = ' num2str(b(1),4) ' + ' ...
                num2str(b(2),4) 'X + ' num2str(b(3),4) 'X^2'])
    
    % Calculate predicted depths and then RMSE
    dPred       =   b(1) + b(2)*ratio + b(3)*ratio.^2;
    rmse        =   sqrt(mean((d-dPred).^2));

    if showPlot
        % Calibration plot
        figure
        plot(ratio,d,'b.');
        axLim       =   axis;
        hold on
        plot(linspace(axLim(1),axLim(2),200),...
            b(1) + b(2)*linspace(axLim(1),axLim(2),200) + ...
            b(3)*linspace(axLim(1),axLim(2),200).^2,'k-','linewidth',2)
        text(0.1,0.9,char(['d = ' num2str(b(1),4) ' + ' ...
                    num2str(b(2),4) 'X + ' ...
                    num2str(b(3),4) 'X^2'],['R^2 = ' num2str(r2max,3) ...
                    '; RMSE = ' num2str(rmse,4) ' m']),...
                    'units','normalized','fontname','arial','fontsize',12)
        xlabel(['X = log(R' num2str(numWvlMax,3) '/R' num2str(denWvlMax,3) ')'])
        ylabel('Depth (m)')
        title('Quadratic OBRA calibration')

        % Plot the OBRA matrix
        xText       =   450;
        yText       =   650;
        mask                =   quadR2;
        mask(isnan(quadR2)) =   0;
        mask                =   logical(mask);
        figure
        imagesc(wvl,wvl,quadR2,'alphadata',mask); axis image; colormap(jet);
        set(gca,'ydir','rev','clim',[0 1])
        box on; axis square
        set(gca,'fontname','arial','fontsize',12)
        xlabel('\lambda_2 (nm)')
        ylabel('\lambda_1 (nm)')
        title('OBRA: quadratic X vs. d relation','fontsize',13)
        hCbar       =   colorbar('location','EastOutside');
        set(get(hCbar,'title'),'string',{'OBRA','R^2'})
        str =   char(['\lambda_1 = ' num2str(numWvlMax,3) ' nm'],...
                    ['\lambda_2 = ' num2str(denWvlMax,3) ' nm'],...
                    ['d = ' num2str(b(1),4) ' + ' num2str(b(2),4) 'X' ...
                     ' + ' num2str(b(3),4) '*X^2'],...
                    ['R^2 = ' num2str(r2max,3) '; RMSE = ' num2str(rmse,3) ' m']);
        text(xText,yText,str,'fontsize',12,'fontname','arial','horizontalalignment','left',...
                'verticalalignment','top');
    end

    % Put these results in an output structure array
    QuadObra=   struct( 'numWvl',numWvlMax,'denWvl',denWvlMax,...
                        'intercept',b(1),'linTerm',b(2),...
                        'quadTerm',b(3),...
                        'R2',r2max,'rmse',rmse);
else
    QuadObra=   [];
end

                
%%  3) Exponential
if fitFlags(3)
    [r2max,iMax]=   max(expR2(:));
    % This will give you the corresponding matrix indices
    [i,j]       =   ind2sub(size(expR2),iMax);
    numWvlMax   =   numWvl(i,j);
    denWvlMax   =   denWvl(i,j);

    % Repeat the fit rather than storing all the coefficients for band ratios we'll
    % never use
    ratio       =   log(spectra(:,wvl==numWvlMax)./spectra(:,wvl==denWvlMax));
    b           =   regress(log(d),[ones(size(ratio)) ratio]);
    a           =   exp(b(1));
    c           =   b(2);
    disp('  ')
    disp('EXPONENTIAL OBRA')
    disp(['     Maximum R-squared: ' num2str(r2max)])
    disp(['     Optimal band ratio: log(R' num2str(numWvlMax) '/R' num2str(denWvlMax) ')'])
    disp(['     X vs. d relation: d = ' num2str(a,4) 'e^{' ...
                num2str(c,4) 'X}'])

    % Calculate predicted depths and then RMSE
    dPred       =   a*exp(c*ratio);
    rmse        =   sqrt(mean((d-dPred).^2));

    if showPlot
        % Calibration plot
        figure
        plot(ratio,d,'b.');
        axLim       =   axis;
        hold on
        plot(linspace(axLim(1),axLim(2),200),...
           a*exp(c*linspace(axLim(1),axLim(2),200)),'k-','linewidth',2)
        axis(axLim);
        text(0.1,0.9,char(['d = ' num2str(a,4) 'e^{' ...
                    num2str(c,4) 'X}'],['R^2 = ' num2str(r2max,3) ...
                    '; RMSE = ' num2str(rmse,4) ' m']),...
                    'units','normalized','fontname','arial','fontsize',12)
        xlabel(['X = log(R' num2str(numWvlMax,3) '/R' num2str(denWvlMax,3) ')'])
        ylabel('Depth (m)')
        title('Exponential OBRA calibration')

        % Plot the OBRA matrix
        xText       =   450;
        yText       =   650;
        mask                =   expR2;
        mask(isnan(expR2))  =   0;
        mask                =   logical(mask);
        figure
        imagesc(wvl,wvl,expR2,'alphadata',mask); axis image; colormap(jet);
        set(gca,'ydir','rev','clim',[0 1])
        box on; axis square
        set(gca,'fontname','arial','fontsize',12)
        xlabel('\lambda_2 (nm)')
        ylabel('\lambda_1 (nm)')
        title('OBRA: exponential X vs. d relation','fontsize',13)
        hCbar       =   colorbar('location','EastOutside');
        set(get(hCbar,'title'),'string',{'OBRA','R^2'})
        str =   char(['\lambda_1 = ' num2str(numWvlMax,3) ' nm'],...
                    ['\lambda_2 = ' num2str(denWvlMax,3) ' nm'],...
                    ['d = ' num2str(a,4) 'e^{' num2str(c,4) 'X}'],...
                    ['R^2 = ' num2str(r2max,3) '; RMSE = ' num2str(rmse,3) ' m']);
        text(xText,yText,str,'fontsize',12,'fontname','arial','horizontalalignment','left',...
                'verticalalignment','top');
    end

    % Put these results in an output structure array
    ExpObra     =   struct( 'numWvl',numWvlMax,'denWvl',denWvlMax,...
                        'constCoeff',a,'expCoeff',c,...
                        'R2',r2max,'rmse',rmse);
else
    ExpObra =   [];
end
                
                
%%  4) Power Law
if fitFlags(4)
    [r2max,iMax]=   max(powerR2(:));
    disp('  ')
    % This will give you the corresponding matrix indices
    [i,j]       =   ind2sub(size(powerR2),iMax);
    numWvlMax   =   numWvl(i,j);
    denWvlMax   =   denWvl(i,j);

    % Repeat the fit rather than storing all the coefficients for band ratios we'll
    % never use
    ratio       =   log(spectra(:,wvl==numWvlMax)./spectra(:,wvl==denWvlMax));
    b           =   regress(log(d),[ones(size(ratio)) log(ratio)]);
    a           =   exp(b(1));
    c           =   b(2);
    disp('POWER LAW OBRA')
    disp(['     Maximum R-squared: ' num2str(r2max)])
    disp(['     Optimal band ratio: log(R' num2str(numWvlMax) '/R' num2str(denWvlMax) ')'])
    disp(['     X vs. d relation: d = ' num2str(a,4) 'X^{' ...
                num2str(c,4) '}'])

    % Calculate predicted depths and then RMSE
    dPred       =   a*ratio.^c;
    rmse        =   sqrt(mean((d-dPred).^2));

    if showPlot
        % Calibration plot
        figure
        plot(ratio,d,'b.');
        axLim       =   axis;
        hold on
        plot(linspace(axLim(1),axLim(2),200),...
           a*linspace(axLim(1),axLim(2),200).^c,'k-','linewidth',2)
        axis(axLim);
        text(0.1,0.9,char(['d = ' num2str(a,4) 'X^{' ...
                    num2str(c,4) '}'],['R^2 = ' num2str(r2max,3) ...
                    '; RMSE = ' num2str(rmse,4) ' m']),...
                    'units','normalized','fontname','arial','fontsize',12)
        xlabel(['X = log(R' num2str(numWvlMax,3) '/R' num2str(denWvlMax,3) ')'])
        ylabel('Depth (m)')
        title('Power Law OBRA calibration')

        % Plot the OBRA matrix
        xText       =   450;
        yText       =   650;
        mask                =   powerR2;
        mask(isnan(powerR2))=   0;
        mask                =   logical(mask);
        figure
        imagesc(wvl,wvl,powerR2,'alphadata',mask); axis image; colormap(jet);
        set(gca,'ydir','rev','clim',[0 1])
        box on; axis square
        set(gca,'fontname','arial','fontsize',12)
        xlabel('\lambda_2 (nm)')
        ylabel('\lambda_1 (nm)')
        title('OBRA: power law X vs. d relation','fontsize',13)
        hCbar       =   colorbar('location','EastOutside');
        set(get(hCbar,'title'),'string',{'OBRA','R^2'})
        str =   char(['\lambda_1 = ' num2str(numWvlMax,3) ' nm'],...
                    ['\lambda_2 = ' num2str(denWvlMax,3) ' nm'],...
                    ['d = ' num2str(a,4) 'X^{' num2str(c,4) '}'],...
                    ['R^2 = ' num2str(r2max,3) '; RMSE = ' num2str(rmse,3) ' m']);
        text(xText,yText,str,'fontsize',12,'fontname','arial','horizontalalignment','left',...
                'verticalalignment','top');
    end

    % Put these results in an output structure array
    PowerObra   =   struct( 'numWvl',numWvlMax,'denWvl',denWvlMax,...
                        'constCoeff',a,'powerCoeff',c,...
                        'R2',r2max,'rmse',rmse);
else
    PowerObra=   [];
end        