function OutOptid = genOptidLin(d,spectra,wvl,dCutMin,dCutMax,dCutIncr)
% Generalized Optimal Band Ratio Analysis (OBRA) of Progressively Truncated Input Depths
%
%% genOptidLin.m:
%   Function for performing a generalized version of Optimal Band Ratio Analysis
%   (OBRA) of Progressively Truncated Input Depths (OPTID), which is a sampling
%   strategy intended to improve spectrally-based depth retrieval.  In this
%   case, linear, quadratic, exponential, and power law models are all
%   considered. Obervations of depth and reflectance exceeding each one of a
%   sequence of depth cutoffs are excluded from the calibration data set used as
%   input to OBRA.  This function performs OBRA with four different functional
%   forms for the data retained for each of the cutoff depths, compiles the OBRA
%   output, and produces a summary plot of the OBRA R-squared value as a
%   function of cutoff depth. This approach thus provides a means of inferring
%   the maximum detectable depth directly from an image as an inflection point
%   on the OBRA R-squared vs. cutoff depth plot.  A second plot showing the
%   numerator and denominator wavelengths of the optimal band ratio as a
%   function of cutoff depth is also produced.
%
%% SYNTAX:
%   OutOptid    =   genOptidLin(d,spectra,wvl,dCutMin,dCutMax,dCutIncr);
%
%% INPUTS:
%   d           =   nData X 1 vector of all depths available for calibration
%   spectra     =   nData X nWvl matrix of corresponding reflectance spectra
%                   with each spectrum in a row and wavelengths as the columns
%   wvl         =   nWvl X 1 vector of wavelengths for reflectance spectra
%   dCutMin     =   scalar specifying the minimum cutoff depth in meters
%   dCutMax     =   scalar specifying the maximum cutoff depth in meters
%   dCutIncr    =   scalar specifying the increment between cutoff depths, in
%                   meters, specified as a negative number such as -0.05 m for 5
%                   cm increments
%
%% OUTPUTS:
%   OutOptid    =   nCutoffs X 1 output structure array with the following fields:
%   .dCut       =   vector of depth cutoffs in meters
%   .iInclude   =   vector of indices into the original calibration data set for
%                   those observations that were retained for each cutoff depth
%   .LinObra    =   structure array with OBRA results for linear model; see help
%                   for genObraLin.m
%   .linR2      =   X vs. d regression R-squared value for the optimal band
%                   ratio for the linear model fit
%   .linNumWvl  =   numerator wavelength for the optimal band ratio (linear fit)
%   .linDenWvl  =   denominator wavelength for the optimal band ratio (linear fit)
%   .QuadObra   =   structure array with OBRA results for quadratic model; see help
%                   for genObraLin.m
%   .quadR2     =   X vs. d regression R-squared value for the optimal band
%                   ratio for the quadratic model fit
%   .quadNumWvl =   numerator wavelength for the optimal band ratio (quadratic fit)
%   .quadDenWvl =   denominator wavelength for the optimal band ratio (quadratic fit)
%   .ExpObra    =   structure array with OBRA results for exponential model; see help
%                   for genObraLin.m
%   .expR2     =   X vs. d regression R-squared value for the optimal band
%                   ratio for the exponential model fit
%   .expNumWvl =   numerator wavelength for the optimal band ratio (exponential fit)
%   .expDenWvl =   denominator wavelength for the optimal band ratio (exponential fit)
%   .PowerObra =   structure array with OBRA results for power law model; see help
%                   for genObraLin.m
%   .powerR2     =   X vs. d regression R-squared value for the optimal band
%                   ratio for the power model fit
%   .powerNumWvl =   numerator wavelength for the optimal band ratio (power fit)
%   .powerDenWvl =   denominator wavelength for the optimal band ratio (power fit)
%   Summary plot of the OBRA R-squared value as a function of cutoff depth
%   A second plot showing the numerator and denominator wavelengths of the
%   optimal band ratio as a function of cutoff depth
%
%% NOTES:
% > Calls genObraLin.m to perform generalized OBRA using linearized forms of the
%   exponential and power law models
% > This approach thus provides a means of inferring the maximum detectable
%   depth directly from an image as an inflection point on the OBRA R-squared 
%   vs. cutoff depth plot.
% > See also stratifiedObra.m, which implements the related approach: Stratified
%   Optimal Band Ratio Analysis (SOBRA)
% > Prototype code developed for the Deschutes River - see
%       D:\Deschutes\RSdata\Depth\DRcasiDepthExhaustive.m
%
%% FUNCTION SUMMARY:
%   OutOptid    =   genOptidLin(d,spectra,wvl,dCutMin,dCutMax,dCutIncr);

%% CREDITS:
% Carl J. Legleiter - cjl@usgs.gov
% 3/27/2018
% C:\Carl\CJLmfiles\RS\genOptid.m


%% Create vector of cutoff depths
% Note that dCutIncr must be a negative number
if dCutIncr > 0
   error('dCutIncr must be a negative number, such as -0.05 for 5 cm increments.') 
end
dCut    =   dCutMax:dCutIncr:dCutMin;


%% Set up output structure array
OutOptid    =   struct('dCut',[],'iInclude',[],...
                       'LinObra',[],'linR2',[],'linNumWvl',[],'linDenWvl',[],...
                       'QuadObra',[],'quadR2',[],'quadNumWvl',[],'quadDenWvl',[],...
                       'ExpObra',[],'expR2',[],'expNumWvl',[],'expDenWvl',[],...
                       'PowerObra',[],'powerR2',[],'powerNumWvl',[],'powerDenWvl',[]);


%% Loop over cutoff depths and perform generalized OBRA at each increment
% Loop over the depth cutoffs, discarding all depths greater than the current
% cutoff to create a new sub-data set, and then performing OBRA for various
% functional forms based on this subset
showPlot    =  0;
% Use the genObra function based on EzyFit for linear, quadratic, and power
fitFlags    =   [1 1 1 1];
for iCut = 1:length(dCut)
    OutOptid(iCut).dCut     =   dCut(iCut);
    OutOptid(iCut).iInclude =   find(d < dCut(iCut));
    tmpSpec                 =   spectra(OutOptid(iCut).iInclude,:);
    tmpD                    =   d(OutOptid(iCut).iInclude);
    [LinObra,QuadObra,ExpObra,PowerObra]    =   genObraLin(tmpSpec,wvl,tmpD,fitFlags,showPlot);
    OutOptid(iCut).LinObra  =   LinObra;
    OutOptid(iCut).linR2    =   OutOptid(iCut).LinObra.R2;
    OutOptid(iCut).linNumWvl=   OutOptid(iCut).LinObra.numWvl;
    OutOptid(iCut).linDenWvl=   OutOptid(iCut).LinObra.denWvl;
    OutOptid(iCut).QuadObra =   QuadObra;
    OutOptid(iCut).quadR2   =   OutOptid(iCut).QuadObra.R2;
    OutOptid(iCut).quadNumWvl=  OutOptid(iCut).QuadObra.numWvl;
    OutOptid(iCut).quadDenWvl=  OutOptid(iCut).QuadObra.denWvl;
    OutOptid(iCut).ExpObra  =   ExpObra;
    OutOptid(iCut).expR2    =   OutOptid(iCut).ExpObra.R2;
    OutOptid(iCut).expNumWvl=   OutOptid(iCut).ExpObra.numWvl;
    OutOptid(iCut).expDenWvl=   OutOptid(iCut).ExpObra.denWvl;
    OutOptid(iCut).PowerObra=   PowerObra;
    OutOptid(iCut).powerR2  =   OutOptid(iCut).PowerObra.R2;
    OutOptid(iCut).powerNumWvl= OutOptid(iCut).PowerObra.numWvl;
    OutOptid(iCut).powerDenWvl= OutOptid(iCut).PowerObra.denWvl;
end


%% Summarize results with plots of OBRA R2 and wavelengths vs. depth cutoff
figure
plot(dCut,[OutOptid.linR2],'k-');
hold on;
plot(dCut,[OutOptid.quadR2],'b-');
plot(dCut,[OutOptid.expR2],'r-');
plot(dCut,[OutOptid.powerR2],'g-');
set(gca,'ylim',[0 1])
legend('Linear','Quadratic','Exponential','Power','location','southeast')
xlabel('Cutoff depth (m)');
ylabel('OBRA R^2');
title({'OPTID: Optimal Band Ratio Analysis (OBRA)', ...
       'of Progressively Truncated Input Depths'})

figure
plot(dCut,[OutOptid.linNumWvl],'b-',dCut,[OutOptid.linDenWvl],'r-');
hold on
plot(dCut,[OutOptid.quadNumWvl],'b:',dCut,[OutOptid.quadDenWvl],'r:');
plot(dCut,[OutOptid.expNumWvl],'b-.',dCut,[OutOptid.expDenWvl],'r-.');
plot(dCut,[OutOptid.powerNumWvl],'b--',dCut,[OutOptid.powerDenWvl],'r--');
xlabel('Cutoff depth (m)')
ylabel('Wavelength (nm)')
title('OPTID: Optimal band ratio wavelengths')
legend( 'Linear: Numerator','Linear: Denominator',...
        'Quadratic: Numerator','Quadratic: Denominator',...
        'Exponential: Numerator','Exponential: Denominator',...
        'Power: Numerator','Power: Denominator',...
        'location','southeast')